module iteration(clkk,first_load,
reg0,
A,
B,
n,
add_s_port_0, val_s_port_0,
add_l_port_1, val_l_port_1,
add_l_port_2, val_l_port_2
);
parameter DW = 15;
input clkk;
input first_load; 
reg [DW:0] counter;
output [DW:0]  add_s_port_0;
output [DW:0]  val_s_port_0;
output [DW:0]  add_l_port_1;
output [DW:0]  val_l_port_1;
output [DW:0]  add_l_port_2;
output [DW:0]  val_l_port_2;
input [DW:0]  reg0;
input [DW:0]  A;
input [DW:0]  B;
input [DW:0]  n;
reg [DW:0] t1;
reg [DW:0] t2;
reg [DW:0] t3;
reg [DW:0] t4;
reg [DW:0] i;

wire [DW:0] add_s_port_0;
wire [DW:0] w_t4;
wire [DW:0] w_t1;
wire [DW:0] w_reg0;
wire [DW:0] w_counter;
wire [DW:0] add_l_port_1;
wire [DW:0] w_t3;
wire [DW:0] w_i;
wire [DW:0] add_l_port_2;
wire [DW:0] w_t2;
wire [DW:0] val_s_port_0;

wire [DW:0] w_Mul1_Right;
wire [DW:0] w_Mul2_Left;
wire [DW:0] w_Mul1_Res;
wire [DW:0] w_Mul1_Left;
wire [DW:0] w_Mul2_Right;
wire [DW:0] w_Mul2_Res;

assign add_s_port_0 =
	counter==6 ? A+i+1  :
	counter==7 ? B+i  :
	add_s_port_0;
assign w_t4 =
	counter==6 ? w_Mul1_Res+val_l_port_1 :
	t4;
assign w_t1 =
	counter==6 ? val_l_port_2 :
	t1;
assign w_reg0 =
	counter==6 ? w_Mul2_Res+val_l_port_2 :
	reg0;
assign w_counter =
	first_load==1 ? 6 :
	counter==7 && i < n-3 ?  6 :
	8;
assign add_l_port_1 =
	counter==7 ? B+i+1-3  :
	add_l_port_1;
assign w_t3 =
	counter==6 ? val_l_port_1 :
	t3;
assign w_i =
	first_load==1 ? 3 :
	counter==7 ? i + 1 :
	i;
assign add_l_port_2 =
	counter==7 ? A+i+3-3  :
	add_l_port_2;
assign w_t2 =
	counter==7 ? reg0 :
	t2;
assign val_s_port_0 =
	counter==6 ? t2 :
	counter==7 ? t4 :
	val_s_port_0;


assign w_Mul1_Right =
	counter==6 ? val_l_port_1 :
	0;
assign w_Mul2_Left =
	counter==6 ? val_l_port_2 :
	0;
assign w_Mul1_Res =
	w_Mul1_Left*w_Mul1_Right;
assign w_Mul1_Left =
	counter==6 ? val_l_port_1 :
	0;
assign w_Mul2_Right =
	counter==6 ? val_l_port_2 :
	0;
assign w_Mul2_Res =
	w_Mul2_Left*w_Mul2_Right;

always @(posedge clkk) 
   begin 

         t4 <= w_t4;
         t1 <= w_t1;
         reg0 <= w_reg0;
         counter <= w_counter;
         t3 <= w_t3;
         i <= w_i;
         t2 <= w_t2;
   end
endmodule

module xram (clk, add_s_port_0, val_s_port_0, add_l_port_1, val_l_port_1, add_l_port_2, val_l_port_2); 
	parameter MEM_SIZE = 20; 
	parameter DW = 15; 
	input [DW:0] add_s_port_0;
	input [DW:0] val_s_port_0;
	input [DW:0] add_l_port_1;
	output [DW:0] val_l_port_1;
	input [DW:0] add_l_port_2;
	output [DW:0] val_l_port_2;
	input clk; 
	reg [DW:0] mem[20:0]; 
	integer i; 
	initial begin 
		for (i = 0; i < MEM_SIZE; i = i + 1) begin
			mem[i] <= 1; 
		end 
	end 
	assign val_l_port_1 = mem[add_l_port_1];
	assign val_l_port_2 = mem[add_l_port_2];
	always @(posedge clk) begin 
		mem[add_s_port_0] = val_s_port_0;
		$display("w mem[%d] == %d; in=%d",add_s_port_0, mem[add_s_port_0], val_s_port_0);
	end
endmodule //xram

module test; 
	parameter DW = 15; 
  	reg reset, clk; 
	wire [DW:0] add_s_port_0;
	wire [DW:0] val_s_port_0;
	wire [DW:0] add_l_port_1;
	wire [DW:0] val_l_port_1;
	wire [DW:0] add_l_port_2;
	wire [DW:0] val_l_port_2;
	xram r1 (clk, add_s_port_0, val_s_port_0, add_l_port_1, val_l_port_1, add_l_port_2, val_l_port_2); 
	iteration instance1 (clk, reset, 0, 0, 0, 0, add_s_port_0, val_s_port_0, add_l_port_1, val_l_port_1, add_l_port_2, val_l_port_2); 
	event reset_trigger; 
	event reset_done_trigger; 
	  initial begin 
	    forever begin 
	      @ (reset_trigger); 
	      @ (negedge clk); 
	      reset = 1; 
	      @ (negedge clk); 
	      reset = 1; 
	      @ (negedge clk); 
	      reset = 1; 
	      @ (negedge clk); 
	      reset = 0; 
	      -> reset_done_trigger;  
	    end 
	  end 
	 initial 
	  begin: DO_RESET 
	    #10 -> reset_trigger; 
	    @ (reset_done_trigger); 
	    @ (negedge clk); 
	    repeat (10) begin 
	      @ (negedge clk); 
	    end 
	  end 
	initial begin 
		clk = 0; 
	end 
	always #5 clk = ~clk; 
	initial #500 $finish; 
endmodule //test 

