! fir_circular: This function illustrates the use of "circular" buffers
! in FIR implementations.  The advantage of circular buffers is that they
! alleviate the need to move data samples around in the delay line (as was
! done in all the functions above).  Most DSP microprocessors implement circular
! buffers in hardware, which allows a single FIR tap can be calculated in a
! single instruction.  That works fine when programming in assembly, but since C
! doesn't have any constructs to represent circular buffers, you need to "fake"
! them in C by adding an extra "if" statement inside the FIR calculation, even if
! the DSP processor provides hardware to implement them without overhead.
!   input    - the input sample data     || ntaps    - the number of taps in the filter
!   h[]      - the FIR coefficient array || z[]      - the FIR delay line array
!   p_state  - the "state" of the filter; initialize this to 1 before the first call
! 	accum 	 - has an answer
in  real input,tmp
out real accum
integer ntaps,state,p_state,n
real h(1:100),z(1:100)
! copy the filter's state to a local
state = p_state
! store input at the beginning of the delay line
z[state]=input
! incr state and check for wrap
state=state+1
if (state >= ntaps) then
	state = 0
endif
! calc FIR and shift data
accum = 0
for ii=ntaps - 1 to n by -1 do
	accum += h(ii) * z(state)
	! incr state and check for wrap
	state=state+1
	if (state >= ntaps) then
    	state = 0
    endif
endfor
! return new state to caller
p_state=state